﻿using System;
using System.Collections.Generic;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace 注水称重系统
{
  /// <summary>
  /// 封装串口调用，处理好跨线程委托调用
  /// </summary>
  /// <remarks>
  /// By 嗷嗷叫的老马 2022-08-19
  /// </remarks>
  class cCommPort
  {
    public delegate void DataRecvEventHandle(string recvData);              //一个委托代理
    public event DataRecvEventHandle eventDataRecv;                         //一个事件

    public delegate void DataRecvByteEventHandle(byte[] recvData);          //一个委托代理
    public event DataRecvByteEventHandle eventByteDataRecv;                 //一个事件

    private SerialPort oComm;
    private Timer oRecvTimer, oSendTimer;
    private Form oFrm;

    byte[] outBuff;
    string outBuffS;
    bool mIsWork;
    /// <summary>
    /// 是否正在进入处理函数
    /// </summary>
    bool mIsInFunction;

    Action actionTimerRecvRestart, actionTimerSendRestart;

    /// <summary>
    /// 初始化，传入UI窗体对象，以及串口对象
    /// </summary>
    /// <param name="UIForm"></param>
    public cCommPort(Form UIForm)
    {
      oFrm = UIForm;

      oRecvTimer = new System.Windows.Forms.Timer();
      oRecvTimer.Tick += oRecvTimer_Tick;
      oRecvTimer.Interval = 50;
      oRecvTimer.Enabled = false;

      oSendTimer = new System.Windows.Forms.Timer();
      oSendTimer.Tick += oSendTimer_Tick;
      oSendTimer.Interval = 50;
      oSendTimer.Enabled = false;

      oComm = new SerialPort();
      oComm.DataReceived += oComm_DataReceived;

      actionTimerRecvRestart = new Action(timerRecvRestart);
      actionTimerSendRestart = new Action(timerSendRestart);
    }

    /// <summary>
    /// 打开串口。
    /// </summary>
    /// <param name="commPort">串口号</param>
    /// <param name="BaudRate">波特率</param>
    /// <param name="parity">校验</param>
    /// <param name="DataBits">数据位长度</param>
    /// <param name="stopBits">停止位长度</param>
    /// <returns>是否成功</returns>
		public bool Open(string commPort, int BaudRate = 9600, Parity parity = Parity.None, int DataBits = 8, StopBits stopBits = StopBits.One)
    {
      bool bRet = false;

      mIsWork = false;

      if (oComm == null)
        return false;

      try
      {
        oComm.PortName = commPort;
        oComm.BaudRate = BaudRate;
        oComm.DataBits = DataBits;
        oComm.Parity = parity;
        oComm.StopBits = stopBits;
        oComm.Open();

        bRet = true;
      }
      catch
      {
        bRet = false;
      }

      mIsWork = bRet;
      return bRet;
    }

    public void Close()
    {
      try
      {
        mIsWork = false;
				do
				{
          Application.DoEvents();
				} while (mIsInFunction);

        oComm.Close();
      }
      catch
      {
      }
    }

    private void oComm_DataReceived(object sender, SerialDataReceivedEventArgs e)
    { //定时器超时法获取串口数据
      //有连续数据到来时会连续重置定时器，直到数据停止发送超过一定时长。
      mIsInFunction = true;
      oFrm.Invoke(actionTimerRecvRestart);    //委托调用下面的timerRecvRestart()
      mIsInFunction = false;
    }

    private void timerRecvRestart()
    {
      oRecvTimer.Enabled = false;       //重置定时器
      oRecvTimer.Enabled = mIsWork;
    }

    private void oRecvTimer_Tick(object sender, EventArgs e)
    {
      oRecvTimer.Enabled = false;

      if (oComm == null)
        return;
      if (oComm.IsOpen != true)
        return;
      if (!mIsWork)
        return;

      if (eventDataRecv != null)        //如果挂接了字符串事件
      {
        oFrm.Invoke(eventDataRecv, oComm.ReadExisting().Trim());
      }
      if (eventByteDataRecv != null)    //如果挂接了字节数组事件
      {
        int _recvLen = oComm.BytesToRead;
        byte[] _buff = new byte[_recvLen];

        oComm.Read(_buff, 0, _recvLen);
        oFrm.Invoke(eventByteDataRecv, _buff);
      }
    }

    public bool SendData(byte[] bDatas)
    {
      if (!mIsWork)
        return false;

      outBuff = bDatas;
      outBuffS = null;
      timerSendRestart();

      if (oComm == null)
        return false;
      if (oComm.IsOpen != true)
        return false;

      return true;
    }

    public bool SendData(string sDatas)
    {
      if (!mIsWork)
        return false;

      outBuff = null;
      outBuffS = sDatas;
      timerSendRestart();

      if (oComm == null)
        return false;
      if (oComm.IsOpen != true)
        return false;

      return true;
    }

    private void timerSendRestart()
    {
      oSendTimer.Enabled = false;       //重置定时器
      oSendTimer.Enabled = mIsWork;
    }

    private void oSendTimer_Tick(object sender, EventArgs e)
    {
      oSendTimer.Enabled = false;

      if (oComm == null)
        return;
      if (oComm.IsOpen != true)
        return;
      if (!mIsWork)
        return;

      if (outBuffS != null)
      {
        oComm.Write(outBuffS);
      }
      if (outBuff != null)
      {
        oComm.Write(outBuff, 0, outBuff.Length);
      }
    }

    public bool IsOpen
    {
      get
      {
        if (oComm == null)
        {
          return false;
        }
        return oComm.IsOpen;
      }
    }
  }
}
